/**********************************************************************
 *
 * querycache.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: querycache.cpp 534 1999-09-07 04:57:43Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.4  1999/09/07 04:57:24  sjboddie
   added gpl notice

   Revision 1.3  1999/01/19 01:38:17  rjmcnab

   Made the source more portable.

   Revision 1.2  1999/01/12 01:51:02  rjmcnab

   Standard header.

   Revision 1.1  1999/01/08 09:02:18  rjmcnab

   Moved from src/library.

 */


#include "querycache.h"




resultcacheel::resultcacheel () 
{
  accessnum = -1;
}



querycache::querycache (int themaxcachesize)
{
  if (themaxcachesize < 1) themaxcachesize = 1;
  
  resultcache = new resultcacheel[themaxcachesize];

  maxcachesize = themaxcachesize;
  nextaccessnum = 1;
}

querycache::~querycache ()
{
  delete [] resultcache;
}

// returns true if the query was found in the cache
// if the query was found then queryresults contains
// the results
bool querycache::find (const queryparamclass &queryparams, 
		       queryresultsclass &queryresults)
{
  int i;

  for (i=0; i < maxcachesize; i++)
    {
      if (resultcache[i].queryparameters == queryparams)
	{
	  queryresults = resultcache[i].queryresults;
	  resultcache[i].accessnum = getnextaccessnum ();
	  return true;
	}
    }

  return false;
}


void querycache::cache (const queryparamclass &queryparams,
	  const queryresultsclass &queryresults)
{
  int i = getfreecachenum();
  resultcache[i].queryparameters = queryparams;
  resultcache[i].queryresults = queryresults;
  resultcache[i].accessnum = getnextaccessnum ();
}

int querycache::getnextaccessnum ()
{
  return nextaccessnum++;
}

int querycache::getfreecachenum ()
{
  int i;
  int minaccessnum = 0;
  int minaccessi = 0;
  
  for (i=0; i < maxcachesize; i++)
    {
      if (resultcache[i].accessnum < minaccessnum) {
	minaccessnum = resultcache[i].accessnum;
	minaccessi = i;
      }
    }

  return minaccessi;
}

